/* wconfig_termconf.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2006 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "wconfig_termconf.hh"
#include "wconfig.h"
#include "worker.h"

TermConfPanel::TermConfPanel( AWindow &basewin, WConfig &baseconfig ) : WConfigPanel( basewin, baseconfig )
{
}

TermConfPanel::~TermConfPanel()
{
}

int TermConfPanel::create()
{
  int res;
  
  res = Panel::create();

  AContainer *ac1 = setContainer( new AContainer( this, 1, 3 ), true );
  ac1->setBorderWidth( 5 );
  ac1->setMinSpace( 5 );
  ac1->setMaxSpace( 5 );
  
  addMultiLineText( catalog.getLocale( 687 ),
                    *ac1,
                    0, 0,
                    NULL, NULL );
  
  ac1->add( new Text( _aguix,
                      0,
                      0,
                      catalog.getLocale( 134 ),
                      1 ), 0, 1, AContainer::CINCWNR );

  sg = (StringGadget*)ac1->add( new StringGadget( _aguix, 0, 0, 30, _baseconfig.getTerminalBin(), 0 ),
                                0, 2, AContainer::CINCW );
  sg->connect( this );

  contMaximize( true );
  return 0;
}

int TermConfPanel::saveValues()
{
  _baseconfig.setTerminalBin( sg->getText() );
  return 0;
}

void TermConfPanel::run( Widget *elem, AGMessage &msg )
{
  if ( msg.type == AG_STRINGGADGET_DEACTIVATE ) {
    if ( msg.stringgadget.sg == sg ) {
      if ( WConfig::isCorrectTerminalBin( sg->getText() ) == false ) {
        // show requester
        std::string textstr = catalog.getLocale( 195 );
        std::string buttonstr = std::string( catalog.getLocale( 313 ) ) + "|" +
                                std::string( catalog.getLocale( 196 ) );
        int erg = request( catalog.getLocale( 125 ), textstr.c_str(), buttonstr.c_str() );
        if ( erg == 0 ) {
          sg->activate();
        } else {
          sg->setText( TERMINAL_BIN );
        }
      }
    }
  }
}
